# @push.rocks/smartdaemon
start scripts as long running daemons and manage them

## Install
To install `@push.rocks/smartdaemon`, run the following command in your terminal:

```bash
npm install @push.rocks/smartdaemon --save
```
This will add `@push.rocks/smartdaemon` to your project's dependencies.

## Usage

`@push.rocks/smartdaemon` is a powerful module designed to help you manage long-running daemons for your applications. Whether you're running background services, scheduled tasks, or any other continuous or long-term operations in a Node.js environment, `smartdaemon` provides an intuitive and flexible API to control these processes.

### Getting Started
First, ensure you import `SmartDaemon` from the package in your TypeScript files:

```typescript
import { SmartDaemon } from '@push.rocks/smartdaemon';
```

### Initialize SmartDaemon
Create an instance of `SmartDaemon`. This instance will be the central point for managing your services.

```typescript
const myDaemon = new SmartDaemon();
```

### Adding a Service
To manage a daemon, you first need to define it as a service. A service in `smartdaemon` is essentially a description of the daemon process you want to manage — including how it should be started, where it should run, and other operational metadata.

Here's an example of adding a simple service that executes a shell command:

```typescript
async function addSampleService() {
  const myService = await myDaemon.addService({
    name: 'mySampleService',
    description: 'A sample service running a long-lived process',
    command: 'node path/to/your/script.js',
    workingDir: '/absolute/path/to/working/directory',
    version: '1.0.0'
  });
  
  await myService.enable();
}
addSampleService();
```

In this example:
- `name`: Unique identifier for the service.
- `description`: A brief explanation of the service.
- `command`: The command that starts your daemon (e.g., node application, shell script).
- `workingDir`: The working directory from which the command will be executed.
- `version`: The version of your service. This can be used for your own versioning and tracking.

### Starting and Stopping Services
After adding and enabling a service, you can control it using the start and stop methods.

```typescript
async function controlService() {
  const myServiceName = 'mySampleService'; // The name of the service you added before
  const myService = await myDaemon.getService(myServiceName);
  
  // To start the service
  await myService.start();

  // To stop the service
  await myService.stop();
}
controlService();
```

### Advanced Management
`smartdaemon` also provides advanced functionalities to manage your daemons effectively, including:
- Checking the status of your services
- Automatically restarting services on failure
- Configuring environment variables or execution parameters

### Working with Systemd
If you're operating on a Linux environment, `smartdaemon` can generate and control services using `systemd`. This integration allows for robust management capabilities, utilizing system-level features for daemon management.

```typescript
// Example: Creating a systemd service
await myDaemon.systemdManager.saveService(myService);
```

### Conclusion
`@push.rocks/smartdaemon` streamlines the management of long-running daemons and background services in Node.js applications. By leveraging its comprehensive API, developers can efficiently control the lifecycle of their services, ensuring their applications run smoothly and reliably.

This module opens up possibilities for more structured and system-integrated application architectures, especially in environments where long-running tasks are essential.

**Note**: Ensure to check permissions and system compatibility, especially when integrating with system-level service managers like `systemd`.

## License and Legal Information

This repository contains open-source code that is licensed under the MIT License. A copy of the MIT License can be found in the [license](license) file within this repository. 

**Please note:** The MIT License does not grant permission to use the trade names, trademarks, service marks, or product names of the project, except as required for reasonable and customary use in describing the origin of the work and reproducing the content of the NOTICE file.

### Trademarks

This project is owned and maintained by Task Venture Capital GmbH. The names and logos associated with Task Venture Capital GmbH and any related products or services are trademarks of Task Venture Capital GmbH and are not included within the scope of the MIT license granted herein. Use of these trademarks must comply with Task Venture Capital GmbH's Trademark Guidelines, and any usage must be approved in writing by Task Venture Capital GmbH.

### Company Information

Task Venture Capital GmbH  
Registered at District court Bremen HRB 35230 HB, Germany

For any legal inquiries or if you require further information, please contact us via email at hello@task.vc.

By using this repository, you acknowledge that you have read this section, agree to comply with its terms, and understand that the licensing of the code does not imply endorsement by Task Venture Capital GmbH of any derivative works.
